@@ -5,7 +5,12 @@ import time
 
 from . import config
 from .cache import CacheHandler
-from .models import OrderList, Order
+
+from .models.orders import OrderList, Order
+from .models.processes import ProcessStatus, ProcessLink
+
+from .constants.reasons import *
+from .constants.transporters import *
 
 class APIEndpoint:
 
@@ -35,6 +40,46 @@ class OrderMethods(APIEndpoint):
 
         status, respJson = self.api.get(url, data)
         return Order().parse(respJson)
+    
+    def cancelItem(self, id, reason):
+        data = { 'orderItems' : [{ 'orderItemId' : id, 'reasonCode' : reason }]}
+
+        url = '{endpoint}/cancellation'.format(endpoint=self.endpoint)
+
+        status, respJson = self.api.put(url, data)
+        return ProcessStatus().parse(respJson)
+    
+    def cancel(self, order, reason):
+
+        orderItems = []
+        for item in order.orderItems:
+            orderItems.append({ 'orderItemId' : item.orderItemId, 'reasonCode' : reason })
+
+        data = { 'orderItems' : orderItems }
+
+        url = '{endpoint}/cancellation'.format(endpoint=self.endpoint)
+
+        status, respJson = self.api.put(url, data)
+        return ProcessStatus().parse(respJson)
+    
+    def ship(self, order, shipmentReference=None, shippingLabelId=None, transporterCode=None, trackAndTrace=None):
+
+        orderItems = []
+        for item in order.orderItems:
+            orderItems.append({ 'orderItemId' : item.orderItemId })
+        
+        data = { 'orderItems' : orderItems }
+
+        if shipmentReference: data['shipmentReference'] = shipmentReference
+        if shippingLabelId: data['shippingLabelId'] = shippingLabelId
+        if transporterCode:
+            data['transport'] = { 'transporterCode' : transporterCode }
+            if trackAndTrace: data['transport']['trackAndTrace'] = trackAndTrace
+        
+        url = '{endpoint}/shipment'.format(endpoint=self.endpoint)
+        
+        status, respJson = self.api.put(url, data)
+        return ProcessStatus().parse(respJson)
 
 class BolAPI:
 
@@ -44,6 +89,7 @@ class BolAPI:
         self.clientSecret = clientSecret
         self.headers = {
             'Accept' : 'application/vnd.retailer.v5+json',
+            'Content-Type' : 'application/vnd.retailer.v5+json',
         }
 
         self.baseUrl = config.DEMO_URL if demo else config.BASE_URL
@@ -53,6 +99,10 @@ class BolAPI:
         self.rateLimitReset = None
 
         self.orders = OrderMethods(self)
+
+
+        self.reasons = Reasons()
+        self.transporters = Transporters()
     
     def setTokenHeader(self, token):
         bearerStr = 'Bearer {token}'.format(token=token)
@@ -83,6 +133,7 @@ class BolAPI:
         }
 
         req = requests.post(config.AUTH_URL, data=None, headers=credentialsHeader)
+      
         status = req.status_code
         response = req.json()
 
@@ -121,6 +172,8 @@ class BolAPI:
             response = requests.get(reqUrl, params=data, headers=headers)
         elif method == 'POST':
             response = requests.post(reqUrl, data=json.dumps(data), headers=headers)
+        elif method == 'PUT':
+            response = requests.put(reqUrl, data=json.dumps(data), headers=headers)
         
         return response
 
@@ -151,4 +204,8 @@ class BolAPI:
     
     def post(self, url, data=None, headers=None):
         status, response = self.request('POST', url, data, headers)
-        return status, response
\ No newline at end of file
+        return status, response
+    
+    def put(self, url, data=None, headers=None):
+        status, response = self.request('PUT', url, data, headers)
+        return status, response
